// This file is generated by Tools/cases_generator/uop_id_generator.py
// from:
//   Python/bytecodes.c
// Do not edit!

#ifndef Py_CORE_UOP_IDS_H
#define Py_CORE_UOP_IDS_H
#ifdef __cplusplus
extern "C" {
#endif

#define _EXIT_TRACE 300
#define _SET_IP 301
#define _BINARY_OP 302
#define _BINARY_OP_ADD_FLOAT 303
#define _BINARY_OP_ADD_INT 304
#define _BINARY_OP_ADD_UNICODE 305
#define _BINARY_OP_EXTEND 306
#define _BINARY_OP_INPLACE_ADD_UNICODE 307
#define _BINARY_OP_MULTIPLY_FLOAT 308
#define _BINARY_OP_MULTIPLY_INT 309
#define _BINARY_OP_SUBSCR_CHECK_FUNC 310
#define _BINARY_OP_SUBSCR_DICT 311
#define _BINARY_OP_SUBSCR_INIT_CALL 312
#define _BINARY_OP_SUBSCR_LIST_INT 313
#define _BINARY_OP_SUBSCR_LIST_SLICE 314
#define _BINARY_OP_SUBSCR_STR_INT 315
#define _BINARY_OP_SUBSCR_TUPLE_INT 316
#define _BINARY_OP_SUBTRACT_FLOAT 317
#define _BINARY_OP_SUBTRACT_INT 318
#define _BINARY_SLICE 319
#define _BUILD_INTERPOLATION BUILD_INTERPOLATION
#define _BUILD_LIST BUILD_LIST
#define _BUILD_MAP BUILD_MAP
#define _BUILD_SET BUILD_SET
#define _BUILD_SLICE BUILD_SLICE
#define _BUILD_STRING BUILD_STRING
#define _BUILD_TEMPLATE BUILD_TEMPLATE
#define _BUILD_TUPLE BUILD_TUPLE
#define _CALL_BUILTIN_CLASS 320
#define _CALL_BUILTIN_FAST 321
#define _CALL_BUILTIN_FAST_WITH_KEYWORDS 322
#define _CALL_BUILTIN_O 323
#define _CALL_INTRINSIC_1 CALL_INTRINSIC_1
#define _CALL_INTRINSIC_2 CALL_INTRINSIC_2
#define _CALL_ISINSTANCE CALL_ISINSTANCE
#define _CALL_KW_NON_PY 324
#define _CALL_LEN 325
#define _CALL_LIST_APPEND CALL_LIST_APPEND
#define _CALL_METHOD_DESCRIPTOR_FAST 326
#define _CALL_METHOD_DESCRIPTOR_FAST_WITH_KEYWORDS 327
#define _CALL_METHOD_DESCRIPTOR_NOARGS 328
#define _CALL_METHOD_DESCRIPTOR_O 329
#define _CALL_NON_PY_GENERAL 330
#define _CALL_STR_1 331
#define _CALL_TUPLE_1 332
#define _CALL_TYPE_1 333
#define _CHECK_AND_ALLOCATE_OBJECT 334
#define _CHECK_ATTR_CLASS 335
#define _CHECK_ATTR_METHOD_LAZY_DICT 336
#define _CHECK_CALL_BOUND_METHOD_EXACT_ARGS 337
#define _CHECK_EG_MATCH CHECK_EG_MATCH
#define _CHECK_EXC_MATCH CHECK_EXC_MATCH
#define _CHECK_FUNCTION 338
#define _CHECK_FUNCTION_EXACT_ARGS 339
#define _CHECK_FUNCTION_VERSION 340
#define _CHECK_FUNCTION_VERSION_INLINE 341
#define _CHECK_FUNCTION_VERSION_KW 342
#define _CHECK_IS_NOT_PY_CALLABLE 343
#define _CHECK_IS_NOT_PY_CALLABLE_KW 344
#define _CHECK_MANAGED_OBJECT_HAS_VALUES 345
#define _CHECK_METHOD_VERSION 346
#define _CHECK_METHOD_VERSION_KW 347
#define _CHECK_PEP_523 348
#define _CHECK_PERIODIC 349
#define _CHECK_PERIODIC_IF_NOT_YIELD_FROM 350
#define _CHECK_RECURSION_REMAINING 351
#define _CHECK_STACK_SPACE 352
#define _CHECK_STACK_SPACE_OPERAND 353
#define _CHECK_VALIDITY 354
#define _COMPARE_OP 355
#define _COMPARE_OP_FLOAT 356
#define _COMPARE_OP_INT 357
#define _COMPARE_OP_STR 358
#define _CONTAINS_OP 359
#define _CONTAINS_OP_DICT 360
#define _CONTAINS_OP_SET 361
#define _CONVERT_VALUE CONVERT_VALUE
#define _COPY COPY
#define _COPY_FREE_VARS COPY_FREE_VARS
#define _CREATE_INIT_FRAME 362
#define _DELETE_ATTR DELETE_ATTR
#define _DELETE_DEREF DELETE_DEREF
#define _DELETE_FAST DELETE_FAST
#define _DELETE_GLOBAL DELETE_GLOBAL
#define _DELETE_NAME DELETE_NAME
#define _DELETE_SUBSCR DELETE_SUBSCR
#define _DEOPT 363
#define _DICT_MERGE DICT_MERGE
#define _DICT_UPDATE DICT_UPDATE
#define _DO_CALL 364
#define _DO_CALL_FUNCTION_EX 365
#define _DO_CALL_KW 366
#define _END_FOR END_FOR
#define _END_SEND END_SEND
#define _ERROR_POP_N 367
#define _EXIT_INIT_CHECK EXIT_INIT_CHECK
#define _EXPAND_METHOD 368
#define _EXPAND_METHOD_KW 369
#define _FATAL_ERROR 370
#define _FORMAT_SIMPLE FORMAT_SIMPLE
#define _FORMAT_WITH_SPEC FORMAT_WITH_SPEC
#define _FOR_ITER 371
#define _FOR_ITER_GEN_FRAME 372
#define _FOR_ITER_TIER_TWO 373
#define _GET_AITER GET_AITER
#define _GET_ANEXT GET_ANEXT
#define _GET_AWAITABLE GET_AWAITABLE
#define _GET_ITER GET_ITER
#define _GET_LEN GET_LEN
#define _GET_YIELD_FROM_ITER GET_YIELD_FROM_ITER
#define _GUARD_BINARY_OP_EXTEND 374
#define _GUARD_CALLABLE_LEN 375
#define _GUARD_CALLABLE_STR_1 376
#define _GUARD_CALLABLE_TUPLE_1 377
#define _GUARD_CALLABLE_TYPE_1 378
#define _GUARD_DORV_NO_DICT 379
#define _GUARD_DORV_VALUES_INST_ATTR_FROM_DICT 380
#define _GUARD_GLOBALS_VERSION 381
#define _GUARD_IS_FALSE_POP 382
#define _GUARD_IS_NONE_POP 383
#define _GUARD_IS_NOT_NONE_POP 384
#define _GUARD_IS_TRUE_POP 385
#define _GUARD_KEYS_VERSION 386
#define _GUARD_NOS_DICT 387
#define _GUARD_NOS_FLOAT 388
#define _GUARD_NOS_INT 389
#define _GUARD_NOS_LIST 390
#define _GUARD_NOS_NULL 391
#define _GUARD_NOS_TUPLE 392
#define _GUARD_NOS_UNICODE 393
#define _GUARD_NOT_EXHAUSTED_LIST 394
#define _GUARD_NOT_EXHAUSTED_RANGE 395
#define _GUARD_NOT_EXHAUSTED_TUPLE 396
#define _GUARD_TOS_ANY_SET 397
#define _GUARD_TOS_DICT 398
#define _GUARD_TOS_FLOAT 399
#define _GUARD_TOS_INT 400
#define _GUARD_TOS_LIST 401
#define _GUARD_TOS_SLICE 402
#define _GUARD_TOS_TUPLE 403
#define _GUARD_TOS_UNICODE 404
#define _GUARD_TYPE_VERSION 405
#define _GUARD_TYPE_VERSION_AND_LOCK 406
#define _IMPORT_FROM IMPORT_FROM
#define _IMPORT_NAME IMPORT_NAME
#define _INIT_CALL_BOUND_METHOD_EXACT_ARGS 407
#define _INIT_CALL_PY_EXACT_ARGS 408
#define _INIT_CALL_PY_EXACT_ARGS_0 409
#define _INIT_CALL_PY_EXACT_ARGS_1 410
#define _INIT_CALL_PY_EXACT_ARGS_2 411
#define _INIT_CALL_PY_EXACT_ARGS_3 412
#define _INIT_CALL_PY_EXACT_ARGS_4 413
#define _INSERT_NULL 414
#define _INSTRUMENTED_FOR_ITER INSTRUMENTED_FOR_ITER
#define _INSTRUMENTED_INSTRUCTION INSTRUMENTED_INSTRUCTION
#define _INSTRUMENTED_JUMP_FORWARD INSTRUMENTED_JUMP_FORWARD
#define _INSTRUMENTED_LINE INSTRUMENTED_LINE
#define _INSTRUMENTED_NOT_TAKEN INSTRUMENTED_NOT_TAKEN
#define _INSTRUMENTED_POP_JUMP_IF_FALSE INSTRUMENTED_POP_JUMP_IF_FALSE
#define _INSTRUMENTED_POP_JUMP_IF_NONE INSTRUMENTED_POP_JUMP_IF_NONE
#define _INSTRUMENTED_POP_JUMP_IF_NOT_NONE INSTRUMENTED_POP_JUMP_IF_NOT_NONE
#define _INSTRUMENTED_POP_JUMP_IF_TRUE INSTRUMENTED_POP_JUMP_IF_TRUE
#define _IS_NONE 415
#define _IS_OP IS_OP
#define _ITER_CHECK_LIST 416
#define _ITER_CHECK_RANGE 417
#define _ITER_CHECK_TUPLE 418
#define _ITER_JUMP_LIST 419
#define _ITER_JUMP_RANGE 420
#define _ITER_JUMP_TUPLE 421
#define _ITER_NEXT_LIST 422
#define _ITER_NEXT_LIST_TIER_TWO 423
#define _ITER_NEXT_RANGE 424
#define _ITER_NEXT_TUPLE 425
#define _JUMP_TO_TOP 426
#define _LIST_APPEND LIST_APPEND
#define _LIST_EXTEND LIST_EXTEND
#define _LOAD_ATTR 427
#define _LOAD_ATTR_CLASS 428
#define _LOAD_ATTR_GETATTRIBUTE_OVERRIDDEN LOAD_ATTR_GETATTRIBUTE_OVERRIDDEN
#define _LOAD_ATTR_INSTANCE_VALUE 429
#define _LOAD_ATTR_METHOD_LAZY_DICT 430
#define _LOAD_ATTR_METHOD_NO_DICT 431
#define _LOAD_ATTR_METHOD_WITH_VALUES 432
#define _LOAD_ATTR_MODULE 433
#define _LOAD_ATTR_NONDESCRIPTOR_NO_DICT 434
#define _LOAD_ATTR_NONDESCRIPTOR_WITH_VALUES 435
#define _LOAD_ATTR_PROPERTY_FRAME 436
#define _LOAD_ATTR_SLOT 437
#define _LOAD_ATTR_WITH_HINT 438
#define _LOAD_BUILD_CLASS LOAD_BUILD_CLASS
#define _LOAD_BYTECODE 439
#define _LOAD_COMMON_CONSTANT LOAD_COMMON_CONSTANT
#define _LOAD_CONST LOAD_CONST
#define _LOAD_CONST_IMMORTAL LOAD_CONST_IMMORTAL
#define _LOAD_CONST_INLINE 440
#define _LOAD_CONST_INLINE_BORROW 441
#define _LOAD_CONST_MORTAL LOAD_CONST_MORTAL
#define _LOAD_DEREF LOAD_DEREF
#define _LOAD_FAST 442
#define _LOAD_FAST_0 443
#define _LOAD_FAST_1 444
#define _LOAD_FAST_2 445
#define _LOAD_FAST_3 446
#define _LOAD_FAST_4 447
#define _LOAD_FAST_5 448
#define _LOAD_FAST_6 449
#define _LOAD_FAST_7 450
#define _LOAD_FAST_AND_CLEAR LOAD_FAST_AND_CLEAR
#define _LOAD_FAST_BORROW 451
#define _LOAD_FAST_BORROW_0 452
#define _LOAD_FAST_BORROW_1 453
#define _LOAD_FAST_BORROW_2 454
#define _LOAD_FAST_BORROW_3 455
#define _LOAD_FAST_BORROW_4 456
#define _LOAD_FAST_BORROW_5 457
#define _LOAD_FAST_BORROW_6 458
#define _LOAD_FAST_BORROW_7 459
#define _LOAD_FAST_BORROW_LOAD_FAST_BORROW LOAD_FAST_BORROW_LOAD_FAST_BORROW
#define _LOAD_FAST_CHECK LOAD_FAST_CHECK
#define _LOAD_FAST_LOAD_FAST LOAD_FAST_LOAD_FAST
#define _LOAD_FROM_DICT_OR_DEREF LOAD_FROM_DICT_OR_DEREF
#define _LOAD_FROM_DICT_OR_GLOBALS LOAD_FROM_DICT_OR_GLOBALS
#define _LOAD_GLOBAL 460
#define _LOAD_GLOBAL_BUILTINS 461
#define _LOAD_GLOBAL_MODULE 462
#define _LOAD_LOCALS LOAD_LOCALS
#define _LOAD_NAME LOAD_NAME
#define _LOAD_SMALL_INT 463
#define _LOAD_SMALL_INT_0 464
#define _LOAD_SMALL_INT_1 465
#define _LOAD_SMALL_INT_2 466
#define _LOAD_SMALL_INT_3 467
#define _LOAD_SPECIAL 468
#define _LOAD_SUPER_ATTR_ATTR LOAD_SUPER_ATTR_ATTR
#define _LOAD_SUPER_ATTR_METHOD LOAD_SUPER_ATTR_METHOD
#define _MAKE_CALLARGS_A_TUPLE 469
#define _MAKE_CELL MAKE_CELL
#define _MAKE_FUNCTION MAKE_FUNCTION
#define _MAKE_WARM 470
#define _MAP_ADD MAP_ADD
#define _MATCH_CLASS MATCH_CLASS
#define _MATCH_KEYS MATCH_KEYS
#define _MATCH_MAPPING MATCH_MAPPING
#define _MATCH_SEQUENCE MATCH_SEQUENCE
#define _MAYBE_EXPAND_METHOD 471
#define _MAYBE_EXPAND_METHOD_KW 472
#define _MONITOR_CALL 473
#define _MONITOR_CALL_KW 474
#define _MONITOR_JUMP_BACKWARD 475
#define _MONITOR_RESUME 476
#define _NOP NOP
#define _POP_EXCEPT POP_EXCEPT
#define _POP_JUMP_IF_FALSE 477
#define _POP_JUMP_IF_TRUE 478
#define _POP_TOP POP_TOP
#define _POP_TOP_LOAD_CONST_INLINE 479
#define _POP_TOP_LOAD_CONST_INLINE_BORROW 480
#define _POP_TWO_LOAD_CONST_INLINE_BORROW 481
#define _PUSH_EXC_INFO PUSH_EXC_INFO
#define _PUSH_FRAME 482
#define _PUSH_NULL PUSH_NULL
#define _PUSH_NULL_CONDITIONAL 483
#define _PY_FRAME_GENERAL 484
#define _PY_FRAME_KW 485
#define _QUICKEN_RESUME 486
#define _REPLACE_WITH_TRUE 487
#define _RESUME_CHECK RESUME_CHECK
#define _RETURN_GENERATOR RETURN_GENERATOR
#define _RETURN_VALUE RETURN_VALUE
#define _SAVE_RETURN_OFFSET 488
#define _SEND 489
#define _SEND_GEN_FRAME 490
#define _SETUP_ANNOTATIONS SETUP_ANNOTATIONS
#define _SET_ADD SET_ADD
#define _SET_FUNCTION_ATTRIBUTE SET_FUNCTION_ATTRIBUTE
#define _SET_UPDATE SET_UPDATE
#define _START_EXECUTOR 491
#define _STORE_ATTR 492
#define _STORE_ATTR_INSTANCE_VALUE 493
#define _STORE_ATTR_SLOT 494
#define _STORE_ATTR_WITH_HINT 495
#define _STORE_DEREF STORE_DEREF
#define _STORE_FAST 496
#define _STORE_FAST_0 497
#define _STORE_FAST_1 498
#define _STORE_FAST_2 499
#define _STORE_FAST_3 500
#define _STORE_FAST_4 501
#define _STORE_FAST_5 502
#define _STORE_FAST_6 503
#define _STORE_FAST_7 504
#define _STORE_FAST_LOAD_FAST STORE_FAST_LOAD_FAST
#define _STORE_FAST_STORE_FAST STORE_FAST_STORE_FAST
#define _STORE_GLOBAL STORE_GLOBAL
#define _STORE_NAME STORE_NAME
#define _STORE_SLICE 505
#define _STORE_SUBSCR 506
#define _STORE_SUBSCR_DICT 507
#define _STORE_SUBSCR_LIST_INT 508
#define _SWAP SWAP
#define _TIER2_RESUME_CHECK 509
#define _TO_BOOL 510
#define _TO_BOOL_BOOL TO_BOOL_BOOL
#define _TO_BOOL_INT TO_BOOL_INT
#define _TO_BOOL_LIST 511
#define _TO_BOOL_NONE TO_BOOL_NONE
#define _TO_BOOL_STR 512
#define _UNARY_INVERT UNARY_INVERT
#define _UNARY_NEGATIVE UNARY_NEGATIVE
#define _UNARY_NOT UNARY_NOT
#define _UNPACK_EX UNPACK_EX
#define _UNPACK_SEQUENCE 513
#define _UNPACK_SEQUENCE_LIST 514
#define _UNPACK_SEQUENCE_TUPLE 515
#define _UNPACK_SEQUENCE_TWO_TUPLE 516
#define _WITH_EXCEPT_START WITH_EXCEPT_START
#define _YIELD_VALUE YIELD_VALUE
#define MAX_UOP_ID 516

#ifdef __cplusplus
}
#endif
#endif /* !Py_CORE_UOP_IDS_H */
