# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 16.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V52
2325
2362
2392
2400
2424
2432
2453
2473
2474
2481
2482
2483
2486
2490
2524
2526
2527
2528
2544
2546
2709
2729
2730
2737
2738
2740
2741
2746
2809
2810
2837
2857
2858
2865
2866
2868
2869
2874
2908
2910
2911
2912
2929
2930
3093
3113
3114
3130
3160
3163
3349
3387
END
