# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 16.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V38
771
773
775
777
778
779
800
801
803
806
813
815
816
817
1548
1549
1563
1565
1567
1568
1600
1601
1611
1622
1648
1649
1792
1806
1807
1867
1869
1872
2144
2155
7672
7673
7674
7675
END
