/* Generated automatically by the program `genconstants'
   from the machine description file `md'.  */

#ifndef GCC_INSN_CONSTANTS_H
#define GCC_INSN_CONSTANTS_H

#define CCDSP_OU_REGNUM 185
#define RETURN_ADDR_REGNUM 31
#define GOT_VERSION_REGNUM 79
#define SET_FCSR_REGNUM 4
#define CCDSP_EF_REGNUM 187
#define CCDSP_CC_REGNUM 186
#define CPRESTORE_SLOT_REGNUM 76
#define PIC_FUNCTION_ADDR_REGNUM 25
#define CCDSP_CA_REGNUM 184
#define CCDSP_PO_REGNUM 182
#define MAX_PIC_BRANCH_LENGTH 100
#define TLS_GET_TP_REGNUM 3
#define CCDSP_SC_REGNUM 183
#define GET_FCSR_REGNUM 2
#define GLOBAL_POINTER_REGNUM 28

enum unspec {
  UNSPEC_LOAD_LEFT = 0,
  UNSPEC_LOAD_RIGHT = 1,
  UNSPEC_STORE_LEFT = 2,
  UNSPEC_STORE_RIGHT = 3,
  UNSPEC_WSBH = 4,
  UNSPEC_DSBH = 5,
  UNSPEC_DSHD = 6,
  UNSPEC_LOAD_LOW = 7,
  UNSPEC_LOAD_HIGH = 8,
  UNSPEC_STORE_WORD = 9,
  UNSPEC_MFHC1 = 10,
  UNSPEC_MTHC1 = 11,
  UNSPEC_GET_FCSR = 12,
  UNSPEC_SET_FCSR = 13,
  UNSPEC_FMIN = 14,
  UNSPEC_FMAX = 15,
  UNSPEC_FRINT = 16,
  UNSPEC_FCLASS = 17,
  UNSPEC_MFHI = 18,
  UNSPEC_MTHI = 19,
  UNSPEC_SET_HILO = 20,
  UNSPEC_LOADGP = 21,
  UNSPEC_COPYGP = 22,
  UNSPEC_MOVE_GP = 23,
  UNSPEC_POTENTIAL_CPRESTORE = 24,
  UNSPEC_CPRESTORE = 25,
  UNSPEC_RESTORE_GP = 26,
  UNSPEC_EH_RETURN = 27,
  UNSPEC_GP = 28,
  UNSPEC_SET_GOT_VERSION = 29,
  UNSPEC_UPDATE_GOT_VERSION = 30,
  UNSPEC_LOAD_CALL = 31,
  UNSPEC_LOAD_GOT = 32,
  UNSPEC_TLS_LDM = 33,
  UNSPEC_TLS_GET_TP = 34,
  UNSPEC_UNSHIFTED_HIGH = 35,
  UNSPEC_ALIGN = 36,
  UNSPEC_CONSTTABLE = 37,
  UNSPEC_CONSTTABLE_END = 38,
  UNSPEC_CONSTTABLE_INT = 39,
  UNSPEC_CONSTTABLE_FLOAT = 40,
  UNSPEC_BLOCKAGE = 41,
  UNSPEC_CLEAR_HAZARD = 42,
  UNSPEC_RDHWR = 43,
  UNSPEC_SYNCI = 44,
  UNSPEC_SYNC = 45,
  UNSPEC_MIPS_CACHE = 46,
  UNSPEC_R10K_CACHE_BARRIER = 47,
  UNSPEC_ERET = 48,
  UNSPEC_DERET = 49,
  UNSPEC_DI = 50,
  UNSPEC_EHB = 51,
  UNSPEC_RDPGPR = 52,
  UNSPEC_COP0 = 53,
  UNSPEC_CALL_ATTR = 54,
  UNSPEC_CASESI_DISPATCH = 55,
  UNSPEC_PROBE_STACK_RANGE = 56,
  UNSPEC_INSN_PSEUDO = 57,
  UNSPEC_JRHB = 58,
  VUNSPEC_SPECULATION_BARRIER = 59,
  UNSPEC_LOONGSON_ALU1_TURN_ENABLED_INSN = 60,
  UNSPEC_LOONGSON_ALU2_TURN_ENABLED_INSN = 61,
  UNSPEC_LOONGSON_FALU1_TURN_ENABLED_INSN = 62,
  UNSPEC_LOONGSON_FALU2_TURN_ENABLED_INSN = 63,
  UNSPEC_COMPARE_AND_SWAP = 64,
  UNSPEC_COMPARE_AND_SWAP_12 = 65,
  UNSPEC_SYNC_OLD_OP = 66,
  UNSPEC_SYNC_NEW_OP = 67,
  UNSPEC_SYNC_NEW_OP_12 = 68,
  UNSPEC_SYNC_OLD_OP_12 = 69,
  UNSPEC_SYNC_EXCHANGE = 70,
  UNSPEC_SYNC_EXCHANGE_12 = 71,
  UNSPEC_MEMORY_BARRIER = 72,
  UNSPEC_ATOMIC_COMPARE_AND_SWAP = 73,
  UNSPEC_ATOMIC_EXCHANGE = 74,
  UNSPEC_ATOMIC_FETCH_OP = 75,
  UNSPEC_MOVE_TF_PS = 76,
  UNSPEC_C = 77,
  UNSPEC_ALNV_PS = 78,
  UNSPEC_CABS = 79,
  UNSPEC_ADDR_PS = 80,
  UNSPEC_CVT_PW_PS = 81,
  UNSPEC_CVT_PS_PW = 82,
  UNSPEC_MULR_PS = 83,
  UNSPEC_ABS_PS = 84,
  UNSPEC_RSQRT1 = 85,
  UNSPEC_RSQRT2 = 86,
  UNSPEC_RECIP1 = 87,
  UNSPEC_RECIP2 = 88,
  UNSPEC_SINGLE_CC = 89,
  UNSPEC_SCC = 90,
  UNSPEC_ADDQ = 91,
  UNSPEC_ADDQ_S = 92,
  UNSPEC_SUBQ = 93,
  UNSPEC_SUBQ_S = 94,
  UNSPEC_ADDSC = 95,
  UNSPEC_ADDWC = 96,
  UNSPEC_MODSUB = 97,
  UNSPEC_RADDU_W_QB = 98,
  UNSPEC_ABSQ_S = 99,
  UNSPEC_PRECRQ_QB_PH = 100,
  UNSPEC_PRECRQ_PH_W = 101,
  UNSPEC_PRECRQ_RS_PH_W = 102,
  UNSPEC_PRECRQU_S_QB_PH = 103,
  UNSPEC_PRECEQ_W_PHL = 104,
  UNSPEC_PRECEQ_W_PHR = 105,
  UNSPEC_PRECEQU_PH_QBL = 106,
  UNSPEC_PRECEQU_PH_QBR = 107,
  UNSPEC_PRECEQU_PH_QBLA = 108,
  UNSPEC_PRECEQU_PH_QBRA = 109,
  UNSPEC_PRECEU_PH_QBL = 110,
  UNSPEC_PRECEU_PH_QBR = 111,
  UNSPEC_PRECEU_PH_QBLA = 112,
  UNSPEC_PRECEU_PH_QBRA = 113,
  UNSPEC_SHLL = 114,
  UNSPEC_SHLL_S = 115,
  UNSPEC_SHRL_QB = 116,
  UNSPEC_SHRA_PH = 117,
  UNSPEC_SHRA_R = 118,
  UNSPEC_MULEU_S_PH_QBL = 119,
  UNSPEC_MULEU_S_PH_QBR = 120,
  UNSPEC_MULQ_RS_PH = 121,
  UNSPEC_MULEQ_S_W_PHL = 122,
  UNSPEC_MULEQ_S_W_PHR = 123,
  UNSPEC_DPAU_H_QBL = 124,
  UNSPEC_DPAU_H_QBR = 125,
  UNSPEC_DPSU_H_QBL = 126,
  UNSPEC_DPSU_H_QBR = 127,
  UNSPEC_DPAQ_S_W_PH = 128,
  UNSPEC_DPSQ_S_W_PH = 129,
  UNSPEC_MULSAQ_S_W_PH = 130,
  UNSPEC_DPAQ_SA_L_W = 131,
  UNSPEC_DPSQ_SA_L_W = 132,
  UNSPEC_MAQ_S_W_PHL = 133,
  UNSPEC_MAQ_S_W_PHR = 134,
  UNSPEC_MAQ_SA_W_PHL = 135,
  UNSPEC_MAQ_SA_W_PHR = 136,
  UNSPEC_BITREV = 137,
  UNSPEC_INSV = 138,
  UNSPEC_REPL_QB = 139,
  UNSPEC_REPL_PH = 140,
  UNSPEC_CMP_EQ = 141,
  UNSPEC_CMP_LT = 142,
  UNSPEC_CMP_LE = 143,
  UNSPEC_CMPGU_EQ_QB = 144,
  UNSPEC_CMPGU_LT_QB = 145,
  UNSPEC_CMPGU_LE_QB = 146,
  UNSPEC_PICK = 147,
  UNSPEC_PACKRL_PH = 148,
  UNSPEC_EXTR_W = 149,
  UNSPEC_EXTR_R_W = 150,
  UNSPEC_EXTR_RS_W = 151,
  UNSPEC_EXTR_S_H = 152,
  UNSPEC_EXTP = 153,
  UNSPEC_EXTPDP = 154,
  UNSPEC_SHILO = 155,
  UNSPEC_MTHLIP = 156,
  UNSPEC_WRDSP = 157,
  UNSPEC_RDDSP = 158,
  UNSPEC_ABSQ_S_QB = 159,
  UNSPEC_ADDU_PH = 160,
  UNSPEC_ADDU_S_PH = 161,
  UNSPEC_ADDUH_QB = 162,
  UNSPEC_ADDUH_R_QB = 163,
  UNSPEC_APPEND = 164,
  UNSPEC_BALIGN = 165,
  UNSPEC_CMPGDU_EQ_QB = 166,
  UNSPEC_CMPGDU_LT_QB = 167,
  UNSPEC_CMPGDU_LE_QB = 168,
  UNSPEC_DPA_W_PH = 169,
  UNSPEC_DPS_W_PH = 170,
  UNSPEC_MADD = 171,
  UNSPEC_MADDU = 172,
  UNSPEC_MSUB = 173,
  UNSPEC_MSUBU = 174,
  UNSPEC_MUL_PH = 175,
  UNSPEC_MUL_S_PH = 176,
  UNSPEC_MULQ_RS_W = 177,
  UNSPEC_MULQ_S_PH = 178,
  UNSPEC_MULQ_S_W = 179,
  UNSPEC_MULSA_W_PH = 180,
  UNSPEC_MULT = 181,
  UNSPEC_MULTU = 182,
  UNSPEC_PRECR_QB_PH = 183,
  UNSPEC_PRECR_SRA_PH_W = 184,
  UNSPEC_PRECR_SRA_R_PH_W = 185,
  UNSPEC_PREPEND = 186,
  UNSPEC_SHRA_QB = 187,
  UNSPEC_SHRA_R_QB = 188,
  UNSPEC_SHRL_PH = 189,
  UNSPEC_SUBU_PH = 190,
  UNSPEC_SUBU_S_PH = 191,
  UNSPEC_SUBUH_QB = 192,
  UNSPEC_SUBUH_R_QB = 193,
  UNSPEC_ADDQH_PH = 194,
  UNSPEC_ADDQH_R_PH = 195,
  UNSPEC_ADDQH_W = 196,
  UNSPEC_ADDQH_R_W = 197,
  UNSPEC_SUBQH_PH = 198,
  UNSPEC_SUBQH_R_PH = 199,
  UNSPEC_SUBQH_W = 200,
  UNSPEC_SUBQH_R_W = 201,
  UNSPEC_DPAX_W_PH = 202,
  UNSPEC_DPSX_W_PH = 203,
  UNSPEC_DPAQX_S_W_PH = 204,
  UNSPEC_DPAQX_SA_W_PH = 205,
  UNSPEC_DPSQX_S_W_PH = 206,
  UNSPEC_DPSQX_SA_W_PH = 207,
  UNSPEC_LOONGSON_PAVG = 208,
  UNSPEC_LOONGSON_PCMPEQ = 209,
  UNSPEC_LOONGSON_PCMPGT = 210,
  UNSPEC_LOONGSON_PEXTR = 211,
  UNSPEC_LOONGSON_PINSRH = 212,
  UNSPEC_LOONGSON_VINIT = 213,
  UNSPEC_LOONGSON_PMADD = 214,
  UNSPEC_LOONGSON_PMOVMSK = 215,
  UNSPEC_LOONGSON_PMULHU = 216,
  UNSPEC_LOONGSON_PMULH = 217,
  UNSPEC_LOONGSON_PMULU = 218,
  UNSPEC_LOONGSON_PASUBUB = 219,
  UNSPEC_LOONGSON_BIADD = 220,
  UNSPEC_LOONGSON_PSADBH = 221,
  UNSPEC_LOONGSON_PSHUFH = 222,
  UNSPEC_LOONGSON_PUNPCKH = 223,
  UNSPEC_LOONGSON_PUNPCKL = 224,
  UNSPEC_LOONGSON_PADDD = 225,
  UNSPEC_LOONGSON_PSUBD = 226,
  UNSPEC_LOONGSON_DSLL = 227,
  UNSPEC_LOONGSON_DSRL = 228,
  UNSPEC_MSA_ASUB_S = 229,
  UNSPEC_MSA_ASUB_U = 230,
  UNSPEC_MSA_AVE_S = 231,
  UNSPEC_MSA_AVE_U = 232,
  UNSPEC_MSA_AVER_S = 233,
  UNSPEC_MSA_AVER_U = 234,
  UNSPEC_MSA_BCLR = 235,
  UNSPEC_MSA_BCLRI = 236,
  UNSPEC_MSA_BINSL = 237,
  UNSPEC_MSA_BINSLI = 238,
  UNSPEC_MSA_BINSR = 239,
  UNSPEC_MSA_BINSRI = 240,
  UNSPEC_MSA_BNEG = 241,
  UNSPEC_MSA_BNEGI = 242,
  UNSPEC_MSA_BSET = 243,
  UNSPEC_MSA_BSETI = 244,
  UNSPEC_MSA_BRANCH_V = 245,
  UNSPEC_MSA_BRANCH = 246,
  UNSPEC_MSA_CFCMSA = 247,
  UNSPEC_MSA_CTCMSA = 248,
  UNSPEC_MSA_FCAF = 249,
  UNSPEC_MSA_FCLASS = 250,
  UNSPEC_MSA_FCUNE = 251,
  UNSPEC_MSA_FEXDO = 252,
  UNSPEC_MSA_FEXP2 = 253,
  UNSPEC_MSA_FEXUPL = 254,
  UNSPEC_MSA_FEXUPR = 255,
  UNSPEC_MSA_FFQL = 256,
  UNSPEC_MSA_FFQR = 257,
  UNSPEC_MSA_FLOG2 = 258,
  UNSPEC_MSA_FRCP = 259,
  UNSPEC_MSA_FRINT = 260,
  UNSPEC_MSA_FRSQRT = 261,
  UNSPEC_MSA_FSAF = 262,
  UNSPEC_MSA_FSEQ = 263,
  UNSPEC_MSA_FSLE = 264,
  UNSPEC_MSA_FSLT = 265,
  UNSPEC_MSA_FSNE = 266,
  UNSPEC_MSA_FSOR = 267,
  UNSPEC_MSA_FSUEQ = 268,
  UNSPEC_MSA_FSULE = 269,
  UNSPEC_MSA_FSULT = 270,
  UNSPEC_MSA_FSUN = 271,
  UNSPEC_MSA_FSUNE = 272,
  UNSPEC_MSA_FTINT_S = 273,
  UNSPEC_MSA_FTINT_U = 274,
  UNSPEC_MSA_FTQ = 275,
  UNSPEC_MSA_MADD_Q = 276,
  UNSPEC_MSA_MADDR_Q = 277,
  UNSPEC_MSA_MSUB_Q = 278,
  UNSPEC_MSA_MSUBR_Q = 279,
  UNSPEC_MSA_MUL_Q = 280,
  UNSPEC_MSA_MULR_Q = 281,
  UNSPEC_MSA_NLOC = 282,
  UNSPEC_MSA_SAT_S = 283,
  UNSPEC_MSA_SAT_U = 284,
  UNSPEC_MSA_SLD = 285,
  UNSPEC_MSA_SLDI = 286,
  UNSPEC_MSA_SPLAT = 287,
  UNSPEC_MSA_SPLATI = 288,
  UNSPEC_MSA_SRAR = 289,
  UNSPEC_MSA_SRARI = 290,
  UNSPEC_MSA_SRLR = 291,
  UNSPEC_MSA_SRLRI = 292,
  UNSPEC_MSA_SUBS_S = 293,
  UNSPEC_MSA_SUBS_U = 294,
  UNSPEC_MSA_SUBSUU_S = 295,
  UNSPEC_MSA_SUBSUS_U = 296,
  UNSPEC_MSA_VSHF = 297,
  UNSPEC_ADDRESS_FIRST = 298
};
#define NUM_UNSPEC_VALUES 299
extern const char *const unspec_strings[];

enum processor {
  PROCESSOR_R3000 = 0,
  PROCESSOR_4KC = 1,
  PROCESSOR_4KP = 2,
  PROCESSOR_5KC = 3,
  PROCESSOR_5KF = 4,
  PROCESSOR_20KC = 5,
  PROCESSOR_24KC = 6,
  PROCESSOR_24KF2_1 = 7,
  PROCESSOR_24KF1_1 = 8,
  PROCESSOR_74KC = 9,
  PROCESSOR_74KF2_1 = 10,
  PROCESSOR_74KF1_1 = 11,
  PROCESSOR_74KF3_2 = 12,
  PROCESSOR_LOONGSON_2E = 13,
  PROCESSOR_LOONGSON_2F = 14,
  PROCESSOR_GS464 = 15,
  PROCESSOR_GS464E = 16,
  PROCESSOR_GS264E = 17,
  PROCESSOR_M4K = 18,
  PROCESSOR_OCTEON = 19,
  PROCESSOR_OCTEON2 = 20,
  PROCESSOR_OCTEON3 = 21,
  PROCESSOR_R3900 = 22,
  PROCESSOR_R6000 = 23,
  PROCESSOR_R4000 = 24,
  PROCESSOR_R4100 = 25,
  PROCESSOR_R4111 = 26,
  PROCESSOR_R4120 = 27,
  PROCESSOR_R4130 = 28,
  PROCESSOR_R4300 = 29,
  PROCESSOR_R4600 = 30,
  PROCESSOR_R4650 = 31,
  PROCESSOR_R4700 = 32,
  PROCESSOR_R5000 = 33,
  PROCESSOR_R5400 = 34,
  PROCESSOR_R5500 = 35,
  PROCESSOR_R5900 = 36,
  PROCESSOR_R7000 = 37,
  PROCESSOR_R8000 = 38,
  PROCESSOR_R9000 = 39,
  PROCESSOR_R10000 = 40,
  PROCESSOR_SB1 = 41,
  PROCESSOR_SB1A = 42,
  PROCESSOR_SR71000 = 43,
  PROCESSOR_XLR = 44,
  PROCESSOR_XLP = 45,
  PROCESSOR_P5600 = 46,
  PROCESSOR_M5100 = 47,
  PROCESSOR_I6400 = 48,
  PROCESSOR_P6600 = 49
};
#define NUM_PROCESSOR_VALUES 50
extern const char *const processor_strings[];

#endif /* GCC_INSN_CONSTANTS_H */
