/* This file is automatically generated.
   This file selects the right generated file of `__stub_FUNCTION' macros
   based on the architecture being compiled for.  */

#include <sgidefs.h>

#if !defined(__mips_nan2008) && defined(__mips_soft_float) && (_MIPS_SIM == _MIPS_SIM_ABI32)
# include <gnu/stubs-o32_soft.h>
#endif
#if !defined(__mips_nan2008) && defined(__mips_hard_float) && (_MIPS_SIM == _MIPS_SIM_ABI32)
# include <gnu/stubs-o32_hard.h>
#endif
#if defined(__mips_nan2008) && defined(__mips_soft_float) && (_MIPS_SIM == _MIPS_SIM_ABI32)
# include <gnu/stubs-o32_soft_2008.h>
#endif
#if defined(__mips_nan2008) && defined(__mips_hard_float) && (_MIPS_SIM == _MIPS_SIM_ABI32)
# include <gnu/stubs-o32_hard_2008.h>
#endif
#if !defined(__mips_nan2008) && defined(__mips_soft_float) && (_MIPS_SIM == _MIPS_SIM_NABI32)
# include <gnu/stubs-n32_soft.h>
#endif
#if !defined(__mips_nan2008) && defined(__mips_hard_float) && (_MIPS_SIM == _MIPS_SIM_NABI32)
# include <gnu/stubs-n32_hard.h>
#endif
#if defined(__mips_nan2008) && defined(__mips_soft_float) && (_MIPS_SIM == _MIPS_SIM_NABI32)
# include <gnu/stubs-n32_soft_2008.h>
#endif
#if defined(__mips_nan2008) && defined(__mips_hard_float) && (_MIPS_SIM == _MIPS_SIM_NABI32)
# include <gnu/stubs-n32_hard_2008.h>
#endif
#if !defined(__mips_nan2008) && defined(__mips_soft_float) && (_MIPS_SIM == _MIPS_SIM_ABI64)
# include <gnu/stubs-n64_soft.h>
#endif
#if !defined(__mips_nan2008) && defined(__mips_hard_float) && (_MIPS_SIM == _MIPS_SIM_ABI64)
# include <gnu/stubs-n64_hard.h>
#endif
#if defined(__mips_nan2008) && defined(__mips_soft_float) && (_MIPS_SIM == _MIPS_SIM_ABI64)
# include <gnu/stubs-n64_soft_2008.h>
#endif
#if defined(__mips_nan2008) && defined(__mips_hard_float) && (_MIPS_SIM == _MIPS_SIM_ABI64)
# include <gnu/stubs-n64_hard_2008.h>
#endif
